// Copyright 2024 Pierre Lairez
// SPDX-License-Identifier: GPL-3.0-or-later

use traits::interval::RealInterval;

#[cfg(not(target_feature = "avx2"))]
mod avx_unavailable {
    compile_error!("AVX2 is not enabled. This program only support x86-64 with AVX2 extensions.");
}

pub mod box_mat;
pub mod box_vec;
pub mod circuit;
pub mod cli;
pub mod flint;
pub mod foa;
pub mod parser;
pub mod precision_ctx;
pub mod prog;
pub mod projective;
pub mod reckless;
pub mod reckless_extra;
pub mod taylor_model;
pub mod traits;

pub fn dummy(t: reckless::Rif, step_size: f64) {
    let ratios = [1.0, 0.875, 0.75, 0.625];
    let step_sizes = ratios.map(|ratio| step_size * ratio);
    let target_ts = step_sizes.map(|step_size| (t.clone() + reckless::Rif::from(step_size)).sup());
    println!("{:?}", target_ts);
}
