// Copyright 2024 Pierre Lairez
// SPDX-License-Identifier: GPL-3.0-or-later

use lalrpop_util::lalrpop_mod;

pub mod ast;

lalrpop_mod!(pub expr, "/parser/expr.rs");
lalrpop_mod!(pub complex, "/parser/complex.rs");
lalrpop_mod!(pub poly, "/parser/poly.rs");

lazy_static::lazy_static! {
    pub static ref COMPLEX: complex::ComplexParser = complex::ComplexParser::new();
    pub static ref EXPR: expr::ExprParser = expr::ExprParser::new();
    pub(crate) static ref POLY: poly::ExprParser = poly::ExprParser::new();
}

#[cfg(test)]
mod test {

    use super::*;
    use ast::Node::*;

    #[test]
    fn test_poly_parser() {
        assert_eq!(POLY.parse("x").unwrap(), Var("x".into()).into());
        assert_eq!(
            POLY.parse("x + y").unwrap(),
            Add(Var("x".into()).into(), Var("y".into()).into()).into()
        );
        assert_eq!(
            POLY.parse("x + (200*I + y)^2*3 - 4.0e-10*x/6").unwrap(),
            Add(
                Add(
                    Var("x".into()).into(),
                    Mul(
                        Power(
                            Add(
                                Mul(Int("200".into()).into(), ImaginaryUnit.into()).into(),
                                Var("y".into()).into()
                            )
                            .into(),
                            2
                        )
                        .into(),
                        Int("3".into()).into()
                    )
                    .into()
                )
                .into(),
                Neg(Box::new(Mul(
                    Mul(
                        Float(ast::FloatFormat {
                            beforedot: "4".into(),
                            afterdot: Some("0".into()),
                            exponent: Some("-10".into())
                        })
                        .into(),
                        Var("x".into()).into()
                    )
                    .into(),
                    Inverse(Int("6".into()).into()).into()
                ))
                .into())
                .into()
            )
            .into()
        )
    }
}
