use std::cell::RefCell;

thread_local! {
    pub static CTX: RefCell<i64> = const { RefCell::new(53) };
}

pub fn set_prec(prec: i64) {
    CTX.with(|ctx| *ctx.borrow_mut() = prec);
}

pub fn get_prec() -> i64 {
    let mut res = 0;
    CTX.with(|ctx| res = *ctx.borrow());
    res
}

pub trait PrecCtx {
    const MAX_PREC: Option<i64>;

    fn get_prec() -> i64;
    fn set_prec(prec: i64) -> std::result::Result<(), ()>;
}

pub struct FixedPrecCtx {}

impl PrecCtx for FixedPrecCtx {
    const MAX_PREC: Option<i64> = Some(53);

    fn get_prec() -> i64 {
        53
    }

    fn set_prec(x: i64) -> std::result::Result<(), ()> {
        match x <= 53 {
            true => Ok(()),
            false => Err(()),
        }
    }
}

pub struct AdaptivePrecCtx {}

impl PrecCtx for AdaptivePrecCtx {
    const MAX_PREC: Option<i64> = None;

    fn get_prec() -> i64 {
        get_prec()
    }

    fn set_prec(prec: i64) -> std::result::Result<(), ()> {
        match prec >= 53 {
            true => Ok(crate::precision_ctx::set_prec(prec)),
            false => Ok(crate::precision_ctx::set_prec(53)),
        }
    }
}
