// Copyright 2024 Pierre Lairez
// SPDX-License-Identifier: GPL-3.0-or-later

pub mod complex;
mod f64extra;
pub mod real;
use std::arch::asm;

use std::arch::x86_64::*;
use std::cell::Ref;
use std::cell::RefCell;

pub type Rif = real::Interval;
pub type Interval = real::Interval;
pub type Cif = complex::Interval;
pub type ComplexInterval = complex::Interval;

#[inline(always)]
pub fn with_rounding_up<T, F>(f: F) -> T
where
    F: FnOnce() -> T,
{
    // We need to do this in order to keep the f call with the right rounding mode (else the compiler may reorder operations and break this property, see dummy_test)
    #[inline(never)]
    fn run<F, T>(f: F) -> T
    where
        F: FnOnce() -> T,
    {
        f()
    }

    #[allow(deprecated)]
    unsafe {
        let csr = _mm_getcsr();
        let newcsr = _MM_ROUND_UP | _MM_MASK_MASK;
        if csr & newcsr != newcsr {
            _mm_setcsr(_MM_ROUND_UP | _MM_MASK_MASK);
        }

        let ret = run(f);
        // let ret = f();

        if csr & newcsr != newcsr {
            _mm_setcsr(csr);
        }
        return ret;
    }
}

// macro_rules! my_setcsr {
//     ($mxcsr:literal) => {
//         unsafe {
//             asm!(
//                 "lea rsp, [rsp - 4]", // Same as "sub rsp, 8", but does not modify flags.
//                 "vstmxcsr [rsp]",
//                 concat!("mov dword ptr [rsp + 4], ", mxcsr),
//                 "vldmxcsr [rsp + 4]",
//                 "mov {rax}, dword ptr [rsp]",
//                 rax = out(reg) _, // Any 64-bit general-purpose register.
//                 options(pure, nomem, preserves_flags)
//             );
//         }
//     }
// }
