use num_traits::{One, Zero};
use std::ops::{Index, IndexMut};

pub trait Matrix:
    Index<(usize, usize), Output = Self::Coeffs>
    + IndexMut<(usize, usize), Output = Self::Coeffs>
    + Clone
{
    type Coeffs: Clone;

    fn ncols(&self) -> usize;
    fn nrows(&self) -> usize;
    fn from_value(i: usize, j: usize, v: Self::Coeffs) -> Self;
    fn from_vec(nrows: usize, ncols: usize, v: Vec<Self::Coeffs>) -> Self; //TODO: remove first argument
    fn get_unchecked_mut(&mut self, i: usize, j: usize) -> &mut Self::Coeffs {
        &mut self[(i, j)]
    }
    fn zeros(i: usize, j: usize) -> Self
    where
        Self::Coeffs: Zero;
    fn id(n: usize) -> Self
    where
        Self::Coeffs: Zero + One;
}
