// Copyright 2024 Pierre Lairez
// SPDX-License-Identifier: GPL-3.0-or-later

// Trait for C-style operations of big types. The syntax is x.ref_op(y, rop), to match with the x.op(y) of Op trait (opposed to op(rop, x, y) in C)
pub trait MulWithRop<Rhs = Self> {
    type Output;

    fn mul_with_rop(&self, rhs: &Rhs, res: &mut Self::Output);
}

pub trait RefDiv<Rhs> {
    type Output;

    fn ref_div(&self, res: &mut Self::Output, rhs: &Rhs);
}

pub trait RefNeg {
    fn ref_neg(&mut self);
}

impl RefNeg for i64 {
    fn ref_neg(&mut self) {
        *self = -*self
    }
}

pub trait InvMat
where
    Self: Sized,
{
    fn inv_mat(m: &Self) -> anyhow::Result<Self>;
}
