// Copyright 2024 Pierre Lairez
// SPDX-License-Identifier: GPL-3.0-or-later
pub trait TaylorModel {
    type Coeffs;
    type Domain; // BoxAlgebra ?

    fn order(&self) -> usize;
    fn from_vec(dom: &Self::Domain, coeffs: &Vec<Self::Coeffs>) -> Self;
    fn domain(&self) -> Self::Domain;
    fn set_domain(&mut self, dom: &Self::Domain);
    fn set_coeff(&mut self, coeff: Self::Coeffs, i: usize); // TODO: determine if this should eat coeff
    fn get_coeff_mut(&mut self, i: usize) -> &mut Self::Coeffs;
    fn enclosure(&self, dom: &Self::Domain) -> Self::Coeffs;
    fn dominter(&mut self, dom: &Self::Domain);
    fn affine(a: &Self::Coeffs, b: &Self::Coeffs, domain: &Self::Domain) -> Self;
    fn identity(domain: &Self::Domain) -> Self;
}

#[cfg(test)]
mod tests {
    use super::*;
    use crate::reckless::{with_rounding_up, Cif, Rif};
    use crate::taylor_model::tm::Tm5;
    use crate::taylor_model::Tm;

    #[test]
    fn mul() {
        let dom = Rif::new(-2.0, 3.0);
        let t = Tm::<5, Rif, Cif>::identity(&dom);
        let t2 = t * t;
        let t3 = t2 * t;
        let t4 = t3 * t;
        assert_eq!(t4.coeffs[3], Cif::ZERO);
        assert_eq!(t4.coeffs[4], Cif::ONE);

        let t5 = t4 * t;
        dbg!(t4);
        dbg!(t5);
        assert_eq!(t5.coeffs[4], dom.into());

        let t8 = t4 * t4;
        let mut t8ref = t4;
        t8ref *= &t4;
        assert_eq!(t8, t8ref);

        with_rounding_up(|| {
            let mut t8fast = Tm5(t4);
            t8fast *= &Tm5(t4);
            assert_eq!(t8fast.0, t8ref);
        })
    }
}
