use algpath::{
    flint::{Acb, Arb},
    traits::{interval::RealInterval, Interval},
};
use num_traits::identities::{One, Zero};

#[test]
fn from_arb() {
    let x = Arb::square(2.0);
    let y: Acb = x.into();
    assert!(Arb(y.0.real).mag() >= 2.0);
    assert!(Arb(y.0.imag).mag() == 0.0);
}

#[test]
fn mag() {
    let x: Acb = Arb::square(13.0).into();
    assert!(x.mag() >= 13.0 && x.mag() <= 14.0);
    let y: Acb = Acb::square(13.0);
    assert!(y.mag() >= 13.0 && y.mag() <= 14.0);
}

#[test]
fn diam() {
    let mut x: Acb = Acb::square(2.0);
    assert!(x.diam() >= 4.0 && 4.1 >= x.diam());
    x.mut_squash();
    assert!(x.diam() == 0.0);
}

#[test]
fn squash() {
    let mut x: Acb = Arb::square(3.0).into();
    assert!(x.diam() != 0.0);
    assert!(x.squash().diam() == 0.0);
    x.mut_squash();
    assert!(x.diam() == 0.0);
}

#[test]
fn partial_eq() {
    let x: Acb = 0.into(); // 0 + I*0
    let y: Acb = 0.into(); // 0 + I*0
    let z = Acb::square(0.0001); // [+/- 0.0001] + I*[+/- 0,.0001]
    let t: Acb = Arb::from_inf_sup(0.1.into(), 0.2.into()).into(); // [0.15 +/- 0.05] + I*0
    assert!(x == y);
    assert!(x != z);
    assert!(x != t);
}

#[test]
fn one() {
    let mut x: Acb = 1.into();
    assert!(x.is_one());
    x = Acb::one();
    assert!(x.is_one());
    x = 123461278941324.into();
    x.set_one();
    assert!(x.is_one());
}

#[test]
fn zero() {
    let mut x: Acb = 0.into();
    assert!(x.is_zero());
    x = Acb::zero();
    assert!(x.is_zero());
    x = 123461278941324.into();
    x.set_zero();
    assert!(x.is_zero());
}
