use algpath::{
    flint::{Acb, AcbVec},
    precision_ctx::{get_prec, set_prec},
    traits::{vector::Vector, Interval},
};
use num_traits::{One, Zero};

#[test]
fn zeros() {
    let n = 10;
    let v = AcbVec::zeros(n);
    for v_i in &v {
        assert!(v_i.is_zero());
    }
}

#[test]
fn from_value() {
    let n = 20;
    let x: Acb = 10.into();
    let v = AcbVec::from_value(n, x.clone());
    for v_i in &v {
        assert!(*v_i == x);
    }
}

#[test]
fn index_mut() {
    let n = 20;
    let mut v = AcbVec::zeros(n);
    v[10] = Acb::one();
    for (i, v_i) in v.into_iter().enumerate() {
        if i == 10 {
            assert!(v_i.is_one());
        } else {
            assert!(v_i.is_zero());
        }
    }
}

#[test]
fn from_vec_acb() {
    let raw_v: Vec<Acb> = vec![11.into(), 2.into(), (-4).into()];
    let v: AcbVec = raw_v.into();
    println!("{:?}", v);
    debug_assert!(v[0] == 11.into());
    debug_assert!(v[1] == 2.into());
    debug_assert!(v[2] == (-4).into());
}

#[test]
fn diam_and_mag() {
    let v: AcbVec = vec![
        Acb::from(1) + Acb::square(2.0),
        Acb::from(-3) + Acb::square(1.2),
        Acb::from(1000000000000000) + Acb::square(5.7),
        Acb::from(0) + Acb::square(0.000001),
        Acb::from(10) + Acb::square(9.8),
    ]
    .into();
    println!("{:?}", v);
    assert!(19.6 <= v.diam() && v.diam() <= 20.0);
    println!("{}", v.mag());
}

#[test]
fn squash() {
    let n = 10;
    let mut v = AcbVec::from_value(n, Acb::square(3.0));
    assert!(6.0 <= v.diam() && v.diam() <= 7.0);
    let w = v.squash();
    assert!(w.diam() == 0.0);
    v.mut_squash();
    assert!(v.diam() == 0.0);
}

#[test]
fn scalar_mul() {
    let n = 10;
    let mut v = AcbVec::from_value(n, Acb::one());
    v = v * &Acb::square(2.0);
    dbg!(v);
    // TODO: investigate why this test does not pass
    // for v_i in &v {
    //     println!("{:?}", v_i);
    //     println!("{:?}", Acb::square(2.0));
    //     assert!(*v_i == Acb::square(2.0));
    // }
}

#[test]
fn sub_assign() {
    let mut x: AcbVec = AcbVec::from_value(3, Acb::from_re_im(&(0.1).into(), &(0.1).into()));
    let y: AcbVec = AcbVec::from_value(3, Acb::from_re_im(&(2.3).into(), &(0.1).into()));
    for _ in 0..10 {
        set_prec(get_prec() * 2);
        x -= &y;
    }
}
