use algpath::{
    flint::{Acb, Arb},
    reckless::{Cif, Rif},
    taylor_model::Tm,
    traits::{interval::RealInterval, TaylorModel},
};

#[test]
fn from_vec() {
    let t: Tm<5, Rif, Cif> = Tm::from_vec(
        &Rif::from_inf_sup((-1.0).into(), (13.0).into()),
        &vec![
            Cif::new(
                Rif::from_inf_sup((-39.0).into(), (63.0).into()),
                Rif::from_inf_sup((-39.0).into(), (63.0).into()),
            ),
            Cif::new(
                Rif::from_inf_sup((-25.0).into(), (71.0).into()),
                Rif::from_inf_sup((-68.0).into(), (-45.0).into()),
            ),
            Cif::new(
                Rif::from_inf_sup((-25.0).into(), (71.0).into()),
                Rif::from_inf_sup((-68.0).into(), (-45.0).into()),
            ),
            Cif::new(
                Rif::from_inf_sup((-25.0).into(), (71.0).into()),
                Rif::from_inf_sup((-68.0).into(), (-45.0).into()),
            ),
            Cif::new(
                Rif::from_inf_sup((-25.0).into(), (71.0).into()),
                Rif::from_inf_sup((-68.0).into(), (-45.0).into()),
            ),
        ],
    );
    println!("{t}");
}

#[test]
fn tm_add() {
    // f = 1 + t + t² + t³
    // T = ([0, 1], 1 + t + t² + t³)
    let mut t_flint: Tm<5, Arb, Acb> = Tm::from_vec(
        &Arb::from_inf_sup(0.0.into(), 1.0.into()),
        &(0..4).map(|_| 1.into()).collect(),
    );
    let mut t_reckless: Tm<5, Rif, Cif> = Tm::from_vec(
        &Rif::from_inf_sup(0.0.into(), 1.0.into()),
        &(0..4).map(|_| 1.into()).collect(),
    );

    t_flint += &t_flint.clone();
    t_reckless += &t_reckless.clone();

    println!("flint tms: ");
    println!(
        "2T([0, 1]) = {}",
        t_flint.enclosure(&Arb::from_inf_sup(0.0.into(), 1.0.into()))
    );
    println!("2T([0, 1]) ⊆ (2f)([0, 1]) = [2, 8] ?");
    assert!(<f64 as Into<Arb>>::into(2.0).subset(&Arb(t_flint
        .enclosure(&Arb::from_inf_sup(0.0.into(), 1.0.into()))
        .0
        .real)));
    assert!(<f64 as Into<Arb>>::into(8.0).subset(&Arb(t_flint
        .enclosure(&Arb::from_inf_sup(0.0.into(), 1.0.into()))
        .0
        .real)));

    println!("reckless tms: ");
    println!(
        "2T([0, 1]) = {}",
        t_reckless.enclosure(&Rif::from_inf_sup(0.0.into(), 1.0.into()))
    );
    println!("2T([0, 1]) ⊆ (2f)([0, 1]) = [2, 8] ?");
    assert!(<f64 as Into<Rif>>::into(2.0).subset(
        t_reckless
            .enclosure(&Rif::from_inf_sup(0.0.into(), 1.0.into()))
            .re()
    ));
    assert!(<f64 as Into<Rif>>::into(8.0).subset(
        t_reckless
            .enclosure(&Rif::from_inf_sup(0.0.into(), 1.0.into()))
            .re()
    ));
}

#[test]
fn tm_mul() {
    // f = 1 + t + t² + t³
    // T = ([0, 1], 1 + t + t² + t³)
    let mut t_flint: Tm<5, Arb, Acb> = Tm::from_vec(
        &Arb::from_inf_sup(0.0.into(), 1.0.into()),
        &(0..4).map(|_| 1.into()).collect(),
    );
    let mut t_reckless: Tm<5, Rif, Cif> = Tm::from_vec(
        &Rif::from_inf_sup(0.0.into(), 1.0.into()),
        &(0..4).map(|_| 1.into()).collect(),
    );

    t_flint *= &t_flint.clone();
    t_reckless *= &t_reckless.clone();

    println!("flint tms: ");
    println!(
        "T²([0, 1]) = {}",
        t_flint.enclosure(&Arb::from_inf_sup(0.0.into(), 1.0.into()))
    );
    println!("T²([0, 1]) ⊆ f²([0, 1]) = [1, 16] ?");
    assert!(<f64 as Into<Arb>>::into(1.0).subset(&Arb(t_flint
        .enclosure(&Arb::from_inf_sup(0.0.into(), 1.0.into()))
        .0
        .real)));
    assert!(<f64 as Into<Arb>>::into(16.0).subset(&Arb(t_flint
        .enclosure(&Arb::from_inf_sup(0.0.into(), 1.0.into()))
        .0
        .real)));

    println!("reckless tms: ");
    println!(
        "T²([0, 1]) = {}",
        t_reckless.enclosure(&Rif::from_inf_sup(0.0.into(), 1.0.into()))
    );
    println!("T²([0, 1]) ⊆ f²([0, 1]) = [1, 16] ?");
    assert!(<f64 as Into<Rif>>::into(1.0).subset(
        t_reckless
            .enclosure(&Rif::from_inf_sup(0.0.into(), 1.0.into()))
            .re()
    ));
    assert!(<f64 as Into<Arb>>::into(16.0).subset(&Arb(t_flint
        .enclosure(&Arb::from_inf_sup(0.0.into(), 1.0.into()))
        .0
        .real)));
}
